@@ -95,71 +95,6 @@ expandFactors <- function( data, ... ) {
   return(df)
 }
 
-xfun <- function( formula, data = NULL, fun, subset = NULL, split = TRUE, ... ) {
-  
-  # check that formula is a two-sided formula
-  if( class(formula) != "formula" ) stop( "formula input must be a formula") 
-  if( length(formula) < 3) stop( "formula must be two-sided" )
-  
-  # get the list of variables included in the formula
-  vars <- all.vars( formula )  
-  
-  # check that the formula has only one LHS variable
-  if( length(vars) != length(all.vars(formula[-2])) +1 ) stop( "only one outcome variable may be specified" )
-  
-  # if variables are not in data frame, check that they are appropriate and then construct one
-  if( is.null(data) ) {
-    
-    # check that the variables exist
-    okay <- sapply(vars,exists)
-    if ( !all(okay)) stop( paste("unable to find variable(s):",paste(names(okay)[!okay],collapse=", ")) )
-    
-    # place all the variables in a list
-    data<-lapply(vars,get) 
-    
-    # check that the variables are all vectors
-    if(!all(sapply(data,is.vector))) stop( "variables must be vectors" )
-    
-    # check that the vectors are of the same length
-    var.lengths<-sapply(data,length)
-    if(all(var.lengths!=var.lengths[1])) stop( "variables are not of the same length" )
-    
-    # construct data frame
-    data<-as.data.frame(data)
-    names(data)<-vars
-  } else {
-    
-    # check that the variables exist within the data frame
-    
-  }
-  
-  # use tapply to apply fun separately to outcome by group
-  if( is.null(subset) ) { 
-    x <- tapply( data[,vars[1]], data[,vars[-1]], fun, ... )
-  } else {
-    x <- tapply( data[subset,vars[1]], data[subset,vars[-1]], fun, ... ) 
-  }
-  
-  # if the output is mode list & the user has requested a split output, split it
-  if( mode(x) == "list" & split == TRUE )  {
-    
-    # check to see if the contents of x are always vectors
-    is.vector <- apply(x,1:length(dim(x)),function(l){is.vector(l[[1]])})
-    
-    if( all(is.vector) ){ 
-      max.len <- max(apply(x,1:length(dim(x)),function(l){length(l[[1]])}))
-      y <- list()
-      for( i in 1:max.len) y[[i]] <- apply(x,1:length(dim(x)),function(l,ind){l[[1]][ind]}, i )
-      x <- y
-    } else {
-      warning( paste("output of function is not a vector: returning array of lists, not list of arrays" ))
-    }
-    
-  }
-  
-  return(x)
-  
-}
 
 colCopy <- function(x,times, dimnames=NULL ) {
   if( is.null(dimnames) ) dimnames<-list(names(x),character(0)) 
