@@ -15,16 +15,16 @@ U{http://www.opensource.org/licenses/cpl1.0.php}
 
 # UIEs to install that ship with LSR
 class install:
-  device = ['Keyboard', 'IBMSpeech', 'FestivalSpeech']
+  device = ['Keyboard', 'CliqueAudio', 'IBMSpeech', 'FestivalSpeech']
   perk = ['DefaultPerk', 'AltShiftPerk', 'MetacityPerk', 'GaimPerk', 
-           'GTerminalPerk']
+           'GTerminalPerk', 'DeveloperPerk']
   monitor = ['RawEventMonitor','TaskMonitor']
   chooser = []
   
 # classes representing initial profile configuration
 class user:
   name = 'user'
-  device = ['Keyboard', 'IBMSpeech', 'FestivalSpeech']
+  device = ['Keyboard', 'CliqueAudio', 'IBMSpeech', 'FestivalSpeech']
   monitor = []
   chooser = []
   perk = {None : ['DefaultPerk', 'AltShiftPerk'],
@@ -36,6 +36,11 @@ class user:
 class developer(user):
   name = 'developer'
   monitor = ['RawEventMonitor','TaskMonitor']
+  perk = {None : ['DefaultPerk', 'AltShiftPerk', 'DeveloperPerk'],
+           'metacity' : ['MetacityPerk'],
+           'gaim' : ['GaimPerk'],
+           'gnome-terminal' : ['GTerminalPerk']
+          }
 
 # list of all profiles to create
 profiles = [user, developer]
\ No newline at end of file
