@@ -19,7 +19,7 @@ See the LSR man pages for help with using command line parameters.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
+@copyright: Copyright (c) 2005, 2006 IBM Corporation
 @license: Common Public License 1.0
 
 All rights reserved. This program and the accompanying materials are made 
@@ -148,15 +148,12 @@ def associate(options):
   '''
   # make sure the default UIEs have been installed at least once in the past
   initRepository()
-  if None in [options.uie_name, options.uie_kind, options.uie_when,
-              options.profile]:
-    raise ValueError(_('Name, kind, profile, and when are required'))
-  if options.uie_when == UIRegistrar.ONE_TIER and not options.uie_tier:
-    raise ValueError(_('Tier name is required when associating with one tier'))
+  if None in [options.uie_name, options.uie_kind, options.profile]:
+    raise ValueError(_('Name, kind and profile are required'))
   
   reg = UIRegistrar.UIRegistrar()
   reg.associate(options.uie_kind, options.uie_name, options.profile, 
-                options.uie_when, options.uie_tier, options.uie_index)
+                options.uie_tier, options.uie_index)
   return _('A %s with name %s was added to profile %s' % 
            (options.uie_kind, options.uie_name, options.profile))
 
@@ -173,20 +170,34 @@ def disassociate(options):
   '''
   # make sure the default UIEs have been installed at least once in the past
   initRepository()
-  if None in [options.uie_name, options.uie_kind, options.uie_when,
-              options.profile]:
-    raise ValueError(_('Name, kind, profile, and when are required'))
-  if options.uie_when == UIRegistrar.ONE_TIER and not options.uie_tier:
-    raise ValueError(_('Tier name is required when disassociating from one tier'))
+  if None in [options.uie_name, options.uie_kind, options.profile]:
+    raise ValueError(_('Name, kind, and profile are required'))
   
   reg = UIRegistrar.UIRegistrar()
   reg.disassociate(options.uie_kind, options.uie_name, options.profile, 
-                   options.uie_when, options.uie_tier)
+                   options.uie_tier)
   return _('A %s with name %s was removed from profile %s' % 
            (options.uie_kind, options.uie_name, options.profile))
 
 def generate(options):
-  ''' 
+  '''
+  Generates a template file for the given kind of UIE in the current working
+  directory. Installs the template UIE immediately and associates it with the 
+  developer profile. Only requires name, kind, (for a Perk) when, and (for a
+  one-tier Perk) tier.
+  
+  @param options: Options seen on the command line
+  @type options: optparse.Values
+  @return: Description of the action
+  @rtype: string
+  @raise ValueError: When required params are not specified
+  '''
+  options.profile = 'developer'
+  options.uie_path = './%s.py' % options.uie_name
+  return expertGenerate(options)
+
+def expertGenerate(options):
+  '''
   Generates a template file for the given kind of UIE. Installs the template
   UIE immediately. Associates it if the name of a profile is also specified.
   
@@ -196,15 +207,14 @@ def generate(options):
   @rtype: string
   @raise ValueError: When required params are not specified
   '''
+  initRepository()
   if None in [options.uie_name, options.uie_path, options.uie_kind]:
     # check for install errors up front
     raise ValueError(_('Name, path, and kind are required'))
   if options.profile is not None:
     # check for association errors up front
-    if None in [options.profile, options.uie_when]:
-      raise ValueError(_('Name, kind, path, profile, and when are required'))
-    if options.uie_when == UIRegistrar.ONE_TIER and not options.uie_tier:
-      raise ValueError(_('Tier name is required when associating with one tier'))
+    if None in [options.profile]:
+      raise ValueError(_('Name, kind, path, and profile are required'))
   # fill in the template and copy it to the appropriate location
   try:
     temp = file(os.path.join(TEMPLATES, options.uie_kind), 'r').read()
@@ -303,7 +313,7 @@ def say(options):
   '''
   reg = UIRegistrar.UIRegistrar()
   prof = options.profile or 'user'
-  for dev in reg.loadAssociated(UIRegistrar.DEVICE, prof, UIRegistrar.STARTUP):
+  for dev in reg.loadAssociated(UIRegistrar.DEVICE, prof):
     # find the first output device
     if implements(dev, AEOutput.AEOutput):
       # speak the utf-8 encoded string seen on the command line
@@ -313,7 +323,8 @@ def say(options):
       except:
         # try another device
         continue
-      dev.sendStringSync(s)
+      styles = dev.createDistinctStyles(1)
+      dev.sendStringSync(s, styles[0])
       dev.close()
       return _('Said: %s') % s
   return _('No default output device')
@@ -348,8 +359,12 @@ def show(options):
     print 'profile:', name
     for kind in UIRegistrar.ALL_KINDS:
       print '  '+kind
-      for when in UIRegistrar.ALL_TIMES:
-        print '    '+when+':', reg.listAssociated(kind, name, when)
+      if kind in UIRegistrar.STARTUP_KINDS:
+        print '    startup:', reg.listAssociated(kind, name)
+      else:
+        print '    all tiers:', reg.listAssociated(kind, name)
+        print '    one tier:', reg.listAssociated(kind, name,
+                                                  all_tier_names=True)
     print
   
   return _('%d installed UIEs, %d profiles') % (i_count, p_count)
@@ -429,16 +444,15 @@ def initRepository():
     # handle all kinds except for Perks
     for kind in UIRegistrar.ALL_KINDS[:-1]:
       for name in getattr(prof, kind):
-        reg.associate(kind, name, prof.name, UIRegistrar.STARTUP)
+        reg.associate(kind, name, prof.name)
         
     # handle default Perks
     for name in prof.perk.get(None, []):
-      reg.associate(UIRegistrar.PERK, name, prof.name, UIRegistrar.ANY_TIER)
+      reg.associate(UIRegistrar.PERK, name, prof.name)
     # handle Tier specific Perks
     for tier in (t for t in prof.perk if t is not None):
       for name in prof.perk[tier]:
-        reg.associate(UIRegistrar.PERK, name, prof.name, UIRegistrar.ONE_TIER,
-                      tier)
+        reg.associate(UIRegistrar.PERK, name, prof.name, tier)
 
   # only set the version once everything has succeeded
   reg.setVersion(version)
@@ -482,61 +496,49 @@ def parseOptions():
   op = optparse.OptionParser()
   # add all the command line options to the parser
   op.add_option('-k', '--kill', action='callback', callback=getAction,
-                help=_('kill a running instance of LSR'), default=kill)
+                help=LSRConstants.HELP_KILL, default=kill)
   op.add_option('-g', '--generate', action='callback', callback=getAction,
-                help=_('generate a template for a new UIE'), default=generate)
+                help=LSRConstants.HELP_GENERATE, default=generate)
   op.add_option('-i', '--install', action='callback', callback=getAction,
-                help=_('install a UIE'), default=install)
+                help=LSRConstants.HELP_INSTALL, default=install)
   op.add_option('-u', '--uninstall', action='callback', callback=getAction,
-                help=_('uninstall a UIE'), default=uninstall)
+                help=LSRConstants.HELP_UNINSTALL, default=uninstall)
   op.add_option('-a', '--associate', action='callback', callback=getAction,
-                help=_('associate a UIE with a profile'), default=associate)
+                help=LSRConstants.HELP_ASSOCIATE, default=associate)
   op.add_option('-d', '--disassociate', action='callback', callback=getAction,
-                help=_('disassociate a UIE from a profile'), 
-                default=disassociate)
+                help=LSRConstants.HELP_DISASSOCIATE, default=disassociate)
   op.add_option('-c', '--create-profile', action='callback', callback=getAction,
-                help=_('create a new user profile'), default=createProfile)
+                help=LSRConstants.HELP_CREATE_PROFILE, default=createProfile)
   op.add_option('-r', '--remove-profile', action='callback', callback=getAction,
-                help=_('remove a user profile'), default=removeProfile)
+                help=LSRConstants.HELP_REMOVE_PROFLE, default=removeProfile)
   op.add_option('-s', '--show', action='callback', callback=getAction,
-                default=show,
-                help=_('show installed perks, profiles, and associated UIEs'))
+                default=show, help=LSRConstants.HELP_SHOW)
   op.add_option('-p', '--profile', action='store', dest='profile',
-                help=_('name of the profile (required for --create-profile, ' \
-                       '--remove-profile, --associate, --disassociate; ' \
-                       'optional when running LSR or using --say or --generate'))
+                help=LSRConstants.HELP_PROFILE)
   op.add_option('-y', '--say', action='callback', callback=getAction, type='str',
-                help=_('output a string using the default device in the given'
-                        'profile (defaults to user)'), default=say)
+                help=LSRConstants.HELP_SAY, default=say)
   op.add_option('--name', dest='uie_name', action='store',
-                help=_('name of the UIE (required for --generate, --install, '\
-                        '--associate, --uninstall, --disassociate'))
+                help=LSRConstants.HELP_NAME)
   op.add_option('--kind', dest='uie_kind', action='store',
                 choices=UIRegistrar.ALL_KINDS,
-                help=_('kind of UIE: [%s] (required for --generate, --install,'\
-                       '--uninstall, --associate, --disassociate') % \
-                       ', '.join(UIRegistrar.ALL_KINDS))
+                help=LSRConstants.HELP_KIND% ', '.join(UIRegistrar.ALL_KINDS))
   op.add_option('--path', dest='uie_path', action='store', 
-                help=_('path to the UIE (required for --generate, --install)'))
-  op.add_option('--when', dest='uie_when', action='store', 
-                choices=UIRegistrar.ALL_TIMES,
-                help=_('when the UIE is loaded: [%s] (required for '\
-                       '--associate, --disassociate, optional for --generate)' % 
-                       ', '.join(UIRegistrar.ALL_TIMES)))
+                help=LSRConstants.HELP_PATH)
   op.add_option('--index', dest='uie_index', action='store', type='int',
-                help=_('load order index for the UIE (optional for --generate,'\
-                       ' --associate)'))
-  op.add_option('--tier', dest='uie_tier', action='store', type='str',
-                help=_('name of the Tier for the UIE (required for --when=onetier'))
+                help=LSRConstants.HELP_INDEX)
+  op.add_option('--tier', '--app', dest='uie_tier', action='store', type='str',
+                help=LSRConstants.HELP_TIER)
   op.add_option('-l', '--log-level', dest='log_level', action='store',
                 choices=['none', 'debug', 'print', 'info', 'warning', 'error'],
-                help=_('level of log messages: ')+' [debug, print, info, ' \
+                help=LSRConstants.HELP_LOG_LEVEL % '[debug, print, info, ' \
                 'warning, error, critical]')
   op.add_option('--log-channel', dest='log_channels', action='append',
-                help=_('channel of log messages, any of: ')+'[Perk, Tier, ...]')
+                help=LSRConstants.HELP_LOG_CHANNEL % '[Perk, Tier, ...]')
   op.add_option('--log-file', dest='log_file', action='store', type='str',
-                help=_('filename for the log, defaults to stderr if not '\
-                       'specified'))
+                help=LSRConstants.HELP_LOG_FILENAME)
+  op.add_option('--expert-generate', action='callback', 
+                callback=getAction,
+                help=LSRConstants.HELP_EXPERT_GENERATE, default=expertGenerate)
   (options, args) = op.parse_args()
   return options
 
